"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const xjs = require("extrajs");
const Element_class_1 = require("./Element.class");
class xjs_HTMLElement extends Element_class_1.default {
    constructor(node) {
        super(node);
    }
    get node() { return super.node; }
    title(val, this_arg = this) {
        return this.attr('title', val, this_arg);
    }
    lang(val, this_arg = this) {
        return this.attr('lang', val, this_arg);
    }
    dir(val, this_arg = this) {
        return this.attr('dir', val, this_arg);
    }
    hidden(val, this_arg = this) {
        return this.attr('hidden', val, this_arg);
    }
    tabIndex(val, this_arg = this) {
        return this.attr('tabindex', val, this_arg);
    }
    style(prop, value, this_arg = this) {
        return xjs.Object.switch(xjs.Object.typeOf(prop), {
            'object': (prp) => {
                for (let i in prp)
                    this.style(i, prp[i]);
                return this;
            },
            'string': (prp) => {
                if (prp.trim() === '')
                    throw new RangeError('Property name cannot be empty string.');
                return xjs.Object.switch(xjs.Object.typeOf(value), {
                    'function': (val) => this.style(prp, val.call(this_arg)),
                    'string': (val) => (val !== '') ? this.node.style.setProperty(prp, val) || this : this.style(prp, null),
                    'number': (val) => this.node.style.setProperty(prp, val.toString()) || this,
                    'boolean': (val) => this.node.style.setProperty(prp, val.toString()) || this,
                    'null': () => this.node.style.removeProperty(prp).slice(0, 0) || this,
                    'undefined': () => this.node.style.getPropertyValue(prp) || null,
                })(value);
            },
            'null': () => this,
            'undefined': () => this,
        })(prop);
    }
    data(data_attr, value, this_arg = this) {
        return xjs.Object.switch(xjs.Object.typeOf(data_attr), {
            'object': (atr) => {
                for (let i in atr)
                    this.data(i, atr[i]);
                return this;
            },
            'string': (atr) => {
                if (atr.trim() === '')
                    throw new RangeError('Data-Attribute name cannot be empty string.');
                return xjs.Object.switch(xjs.Object.typeOf(value), {
                    'function': (val) => this.data(atr, val.call(this_arg)),
                    'string': (val) => { this.node.dataset[atr] = val; return this; },
                    'number': (val) => { this.node.dataset[atr] = val.toString(); return this; },
                    'boolean': (val) => { this.node.dataset[atr] = val.toString(); return this; },
                    'null': () => { delete this.node.dataset[atr]; return this; },
                    'undefined': () => {
                        const returned = this.node.dataset[atr];
                        return (typeof returned === 'string') ? returned : null;
                    },
                })(value);
            },
            'null': () => this,
            'undefined': () => this,
        })(data_attr);
    }
}
exports.default = xjs_HTMLElement;
